/* Copyright (C) 2008 P.L. Lucas
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, 
 * Boston, MA 02111-1307, USA. 
 */



#include <gtk/gtk.h>
#include <glade/glade.h>
#include <vte/vte.h>
#include <vte/reaper.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "main_xml.h"
#include "postscript_options.h"

static GladeXML *xml;
static GtkWidget *vte;
static VteReaper *vte_reaper;
char *file_name=NULL;

/*************************************************************************************************/
void on_window_delete_event(GtkWidget *widget, gpointer user_data) {
  gtk_main_quit();
}


/*************************************************************************************************/
gint reaper_signal(VteReaper *vte_reaper, gint pid, gint status, gpointer data){
	on_window_delete_event(NULL,NULL);
}


/*************************************************************************************************/
//GtkWidget *vte_new(gchar *widget_name, gchar *string1, gchar *string2,
//                       gint int1, gint int2) {
void vte_new()
{
        GtkWidget *hbox;
	vte=vte_terminal_new();
	vte_reaper=vte_reaper_get();
	hbox=glade_xml_get_widget(xml, "hbox");
	gtk_box_pack_start_defaults (GTK_BOX(hbox), vte);
	gtk_widget_show(vte);
	g_signal_connect(vte_reaper,"child-exited", GTK_SIGNAL_FUNC(reaper_signal), NULL);
	vte_terminal_set_font_from_string(VTE_TERMINAL(vte), "monospace 10");
	vte_terminal_fork_command( VTE_TERMINAL(vte),"gnuplot",NULL, NULL,".",FALSE,FALSE,FALSE);
	//return vte;
}


/*************************************************************************************************/
void on_quit_menuitem_activate(GtkMenuItem *menuitem,gpointer user_data){
	on_window_delete_event(NULL,NULL);
}

/*************************************************************************************************/
void open_filename (GtkWidget *widget, gpointer user_data) {
   GtkWidget *file_selector = (GtkWidget *)user_data;
   const gchar *selected_filename= gtk_file_selection_get_filename (GTK_FILE_SELECTION (file_selector));
   if(file_name!=NULL) free(file_name);
   file_name=(char*)malloc(sizeof(char)*(strlen(selected_filename)+10));
   sprintf(file_name,"load '%s'\n",selected_filename);
   vte_terminal_feed_child(VTE_TERMINAL(vte), file_name, strlen(file_name));
   sprintf(file_name,"%s",selected_filename);
}

/*************************************************************************************************/
void save_filename (GtkWidget *widget, gpointer user_data) {
   GtkWidget *file_selector = (GtkWidget *)user_data;
   const gchar *selected_filename= gtk_file_selection_get_filename (GTK_FILE_SELECTION (file_selector));
   if(file_name!=NULL) free(file_name);
   file_name=(char*)malloc(sizeof(char)*(strlen(selected_filename)+10));
   sprintf(file_name,"save '%s'\n",selected_filename);
   vte_terminal_feed_child(VTE_TERMINAL(vte), file_name, strlen(file_name));
   sprintf(file_name,"%s",selected_filename);
}


/*************************************************************************************************/
void create_file_selection ( void (*callback)(GtkWidget*,gpointer) ) {
   GtkWidget *file_selector;
   /* Create the selector */
   file_selector = gtk_file_selection_new ("Select File...");
   g_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION (file_selector)->ok_button),
                     "clicked",
                     G_CALLBACK (callback),
                     (gpointer) file_selector);	   
   /* Ensure that the dialog box is destroyed when the user clicks a button. */
   g_signal_connect_swapped (GTK_OBJECT (GTK_FILE_SELECTION (file_selector)->ok_button),
                             "clicked",
                             G_CALLBACK (gtk_widget_destroy), 
                             (gpointer) file_selector); 
   g_signal_connect_swapped (GTK_OBJECT (GTK_FILE_SELECTION (file_selector)->cancel_button),
                             "clicked",
                             G_CALLBACK (gtk_widget_destroy),
                             (gpointer) file_selector); 
   /* Display that dialog */
   gtk_widget_show (file_selector);
}

/*************************************************************************************************/
void on_open_menuitem_activate(GtkMenuItem *menuitem,gpointer user_data){
	create_file_selection(open_filename);
}

/*************************************************************************************************/
void on_save_as_menuitem_activate(GtkMenuItem *menuitem,gpointer user_data){
	create_file_selection(save_filename);
}

/*************************************************************************************************/
void on_save_menuitem_activate(GtkMenuItem *menuitem,gpointer user_data){
	if(file_name!=NULL) {
		char *buf=(char*)malloc(sizeof(char)*(strlen(file_name)+10));
		sprintf(buf,"save '%s'\n",file_name);
		vte_terminal_feed_child(VTE_TERMINAL(vte), buf, strlen(buf));
		free(buf);
	} else {
		create_file_selection(save_filename);
	}
}

/*************************************************************************************************/
void on_about_menuitem_activate(GtkMenuItem *menuitem,gpointer user_data){
	GtkWidget *dialog = gtk_message_dialog_new (NULL,
                                  GTK_DIALOG_MODAL,
                                  GTK_MESSAGE_INFO,
				GTK_BUTTONS_CLOSE,
				"Gtk_Gnuplot\nA gnuplot front-end.\n\n"
				"Copyright (C) 2008 P.L. Lucas\n"
				" This program is free software; you can redistribute it and/or modify"
				" it under the terms of the GNU General Public License as published by"
				" the Free Software Foundation; either version 2 of the License, or"
				" (at your option) any later version.\n"
				"\n"
				" This program is distributed in the hope that it will be useful,"
				" but WITHOUT ANY WARRANTY; without even the implied warranty of"
				" MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the"
				" GNU General Public License for more details.\n"
				"\n"
				" You should have received a copy of the GNU General Public License"
				" along with this program; if not, write to the Free Software"
				" Foundation, Inc., 59 Temple Place, Suite 330,"
				" Boston, MA 02111-1307, USA. "
                                  );
	gtk_dialog_run(GTK_DIALOG(dialog));
  	gtk_widget_destroy (dialog);
}

/*************************************************************************************************/
void on_copy_menuitem_activate(GtkMenuItem *menuitem,gpointer user_data){
	vte_terminal_copy_clipboard(VTE_TERMINAL(vte));
}

/*************************************************************************************************/
void on_paste_menuitem_activate(GtkMenuItem *menuitem,gpointer user_data){
	vte_terminal_paste_clipboard(VTE_TERMINAL(vte));
}

/*************************************************************************************************/
void on_to_eps_menuitem_activate(GtkMenuItem *menuitem,gpointer user_data){
	char *command;
	GtkWidget *file_selection=gtk_file_selection_new("Select eps output");
	gint result;
	char term_command[100];
	if(!create_postscript_options(term_command)) {
		gtk_widget_destroy(file_selection);
		return;
	}
	gtk_widget_hide(GTK_FILE_SELECTION(file_selection)->ok_button);
	gtk_widget_hide(GTK_FILE_SELECTION(file_selection)->cancel_button);
	gtk_dialog_add_buttons((GtkDialog *)file_selection,
                                             	  GTK_STOCK_OK,
                                                  GTK_RESPONSE_OK,
                                                  GTK_STOCK_CANCEL,
                                                  GTK_RESPONSE_REJECT,
                                                  NULL);
	result=gtk_dialog_run(GTK_DIALOG(file_selection));
	if( result == GTK_RESPONSE_OK ) {
		const gchar *selected_filename= gtk_file_selection_get_filename (GTK_FILE_SELECTION (file_selection));
		command=(char *)malloc(sizeof(char)*(strlen(selected_filename)+20));
		sprintf(command,"set output \"%s\"\n",selected_filename);
		vte_terminal_feed_child(VTE_TERMINAL(vte), term_command,
			strlen(term_command));
		vte_terminal_feed_child(VTE_TERMINAL(vte), command, strlen(command));
		vte_terminal_feed_child(VTE_TERMINAL(vte), "replot\n", strlen("replot\n"));
		vte_terminal_feed_child(VTE_TERMINAL(vte), "set term X11\n", strlen("set term X11\n"));
		free(command);
	}
	gtk_widget_destroy(file_selection);
}

/*************************************************************************************************/
void on_to_latex_menuitem_activate(GtkMenuItem *menuitem,gpointer user_data){
	create_to_latex_options(vte);
}

/*************************************************************************************************/
void on_plot_menuitem_activate(GtkMenuItem *menuitem,gpointer user_data){
	vte_terminal_feed_child(VTE_TERMINAL(vte), "plot ", strlen("plot "));
	create_plot_options(vte);
}

/*************************************************************************************************/
void on_plot_data_file_menuitem_activate(GtkMenuItem *menuitem,gpointer user_data){
	vte_terminal_feed_child(VTE_TERMINAL(vte), "plot ", strlen("plot "));
	create_plot_datafile_options(vte);
}

/*************************************************************************************************/
void on_plot_3d_menuitem_activate(GtkMenuItem *menuitem,gpointer user_data){
	vte_terminal_feed_child(VTE_TERMINAL(vte), "splot ", strlen("splot "));
	create_plot_options(vte);
}

/*************************************************************************************************/
void on_plot_3d_data_file_menuitem_activate(GtkMenuItem *menuitem,gpointer user_data){
	vte_terminal_feed_child(VTE_TERMINAL(vte), "splot ", strlen("splot "));
	create_plot_datafile_options(vte);
}

/*************************************************************************************************/
void on_set_grid_menuitem_activate(GtkMenuItem *menuitem,gpointer user_data){
	char *c="set grid\n";
	vte_terminal_feed_child(VTE_TERMINAL(vte), c, strlen(c));
}

/*************************************************************************************************/
void on_set_no_grid_menuitem_activate(GtkMenuItem *menuitem,gpointer user_data){
	char *c="unset grid\n";
	vte_terminal_feed_child(VTE_TERMINAL(vte), c, strlen(c));
}

/*************************************************************************************************/
void on_set_polar_menuitem_activate(GtkMenuItem *menuitem,gpointer user_data){
	char *c="set polar\n";
	vte_terminal_feed_child(VTE_TERMINAL(vte), c, strlen(c));
}

/*************************************************************************************************/
void on_set_no_polar_menuitem_activate(GtkMenuItem *menuitem,gpointer user_data){
	char *c="unset polar\n";
	vte_terminal_feed_child(VTE_TERMINAL(vte), c, strlen(c));
}

/*************************************************************************************************/
void on_set_radians_menuitem_activate(GtkMenuItem *menuitem,gpointer user_data){
	char *c="set angle radians\n";
	vte_terminal_feed_child(VTE_TERMINAL(vte), c, strlen(c));
}

/*************************************************************************************************/
void on_set_degrees_menuitem_activate(GtkMenuItem *menuitem,gpointer user_data){
	char *c="set angle degrees\n";
	vte_terminal_feed_child(VTE_TERMINAL(vte), c, strlen(c));
}

/*************************************************************************************************/
void on_set_title_menuitem_activate(GtkMenuItem *menuitem,gpointer user_data){
	char *c="set title \"Put title here\" \"Helvetica,12\"";
	vte_terminal_feed_child(VTE_TERMINAL(vte), c, strlen(c));
}

/*************************************************************************************************/
void on_set_xlabel_menuitem_activate(GtkMenuItem *menuitem,gpointer user_data){
	char *c="set xlabel \"Put label here\" \"Helvetica,12\"";
	vte_terminal_feed_child(VTE_TERMINAL(vte), c, strlen(c));
}

/*************************************************************************************************/
void on_set_ylabel_menuitem_activate(GtkMenuItem *menuitem,gpointer user_data){
	char *c="set ylabel \"Put label here\" \"Helvetica,12\"";
	vte_terminal_feed_child(VTE_TERMINAL(vte), c, strlen(c));
}

/*************************************************************************************************/
void on_set_zlabel_menuitem_activate(GtkMenuItem *menuitem,gpointer user_data){
	char *c="set zlabel \"Put label here\" \"Helvetica,12\"";
	vte_terminal_feed_child(VTE_TERMINAL(vte), c, strlen(c));
}

/*************************************************************************************************/
void on_set_legend_menuitem_activate(GtkMenuItem *menuitem,gpointer user_data){
	char *c="set key on\n";
	vte_terminal_feed_child(VTE_TERMINAL(vte), c, strlen(c));
}

/*************************************************************************************************/
void on_set_no_legend_menuitem_activate(GtkMenuItem *menuitem,gpointer user_data){
	char *c="set key off\n";
	vte_terminal_feed_child(VTE_TERMINAL(vte), c, strlen(c));
}

/*************************************************************************************************/
void on_set_linear_menuitem_activate(GtkMenuItem *menuitem,gpointer user_data){
	char *c="set cntrparam linear\n";
	vte_terminal_feed_child(VTE_TERMINAL(vte), c, strlen(c));
}

/*************************************************************************************************/
void on_set_cubic_spline_menuitem_activate(GtkMenuItem *menuitem,gpointer user_data){
	char *c="set cntrparam cubicspline\n";
	vte_terminal_feed_child(VTE_TERMINAL(vte), c, strlen(c));
}

/*************************************************************************************************/
void on_set_b_spline_contours_menuitem_activate(GtkMenuItem *menuitem,gpointer user_data){
	char *c="set cntrparam bspline\n";
	vte_terminal_feed_child(VTE_TERMINAL(vte), c, strlen(c));
}

/*************************************************************************************************/
void on_set_contours_on_base_menuitem_activate(GtkMenuItem *menuitem,gpointer user_data){
	char *c="set contour base\n";
	vte_terminal_feed_child(VTE_TERMINAL(vte), c, strlen(c));
}

/*************************************************************************************************/
void on_set_contours_on_surfaces_menuitem_activate(GtkMenuItem *menuitem,gpointer user_data){
	char *c="set contour surface\n";
	vte_terminal_feed_child(VTE_TERMINAL(vte), c, strlen(c));
}

/*************************************************************************************************/
void on_set_contours_on_both_menuitem_activate(GtkMenuItem *menuitem,gpointer user_data){
	char *c="set contour both\n";
	vte_terminal_feed_child(VTE_TERMINAL(vte), c, strlen(c));
}

/*************************************************************************************************/
void on_set_no_contours_menuitem_activate(GtkMenuItem *menuitem,gpointer user_data){
	char *c="unset contour\n";
	vte_terminal_feed_child(VTE_TERMINAL(vte), c, strlen(c));
}

/*************************************************************************************************/
void on_set_cartesian_menuitem_activate(GtkMenuItem *menuitem,gpointer user_data){
	char *c="set mapping cartesian\n";
	vte_terminal_feed_child(VTE_TERMINAL(vte), c, strlen(c));
}

/*************************************************************************************************/
void on_set_spherical_menuitem_activate(GtkMenuItem *menuitem,gpointer user_data){
	char *c="set mapping spherical\n";
	vte_terminal_feed_child(VTE_TERMINAL(vte), c, strlen(c));
}

/*************************************************************************************************/
void on_set_cylindrical_menuitem_activate(GtkMenuItem *menuitem,gpointer user_data){
	char *c="set mapping cylindrical\n";
	vte_terminal_feed_child(VTE_TERMINAL(vte), c, strlen(c));
}

/*************************************************************************************************/
void on_set_line_removal_menuitem_activate(GtkMenuItem *menuitem,gpointer user_data){
	char *c="set hidden3d\n";
	vte_terminal_feed_child(VTE_TERMINAL(vte), c, strlen(c));
}

/*************************************************************************************************/
void on_set_no_line_removal_menuitem_activate(GtkMenuItem *menuitem,gpointer user_data){
	char *c="unset hidden3d\n";
	vte_terminal_feed_child(VTE_TERMINAL(vte), c, strlen(c));
}

/*************************************************************************************************/
void on_set_surface_mesh_menuitem_activate(GtkMenuItem *menuitem,gpointer user_data){
	char *c="set surface\n";
	vte_terminal_feed_child(VTE_TERMINAL(vte), c, strlen(c));
}

/*************************************************************************************************/
void on_set_no_surface_mesh_menuitem_activate(GtkMenuItem *menuitem,gpointer user_data){
	char *c="unset surface\n";
	vte_terminal_feed_child(VTE_TERMINAL(vte), c, strlen(c));
}



/*************************************************************************************************/
void on_gnuplot_manual_menuitem_activate(GtkMenuItem *menuitem,gpointer user_data){
	char *c=GNUPLOT_MANUAL;
	vte_terminal_feed_child(VTE_TERMINAL(vte), c, strlen(c));
}

/*************************************************************************************************/
void on_open_button_clicked(GtkWidget *widget, gpointer user_data) {
	create_file_selection(open_filename);
}

/*************************************************************************************************/
void on_replot_button_clicked(GtkMenuItem *menuitem,gpointer user_data){
	vte_terminal_feed_child(VTE_TERMINAL(vte), "replot\n", 7);
}

/*************************************************************************************************/
void on_quit_button_clicked(GtkWidget *widget, gpointer user_data) {
	gtk_main_quit();
}

/*************************************************************************************************/
void on_find_button_clicked(GtkWidget *widget, gpointer user_data) {
	GtkWidget *file_selection=gtk_file_selection_new("Select eps output");
	gint result;
	gtk_widget_hide(GTK_FILE_SELECTION(file_selection)->ok_button);
	gtk_widget_hide(GTK_FILE_SELECTION(file_selection)->cancel_button);
	gtk_dialog_add_buttons((GtkDialog *)file_selection,
                                             	  GTK_STOCK_OK,
                                                  GTK_RESPONSE_OK,
                                                  GTK_STOCK_CANCEL,
                                                  GTK_RESPONSE_REJECT,
                                                  NULL);
	result=gtk_dialog_run(GTK_DIALOG(file_selection));
	if( result == GTK_RESPONSE_OK ) {
		const gchar *selected_filename= gtk_file_selection_get_filename (GTK_FILE_SELECTION (file_selection));
		char *command=(char *)malloc(sizeof(char)*(strlen(selected_filename)+20));
		sprintf(command,"\"%s\"",selected_filename);
		vte_terminal_feed_child(VTE_TERMINAL(vte), command, strlen(command));
		free(command);
	}
	gtk_widget_destroy(file_selection);
}

/*************************************************************************************************/
void on_insert_button_clicked(GtkWidget *widget, gpointer user_data) {
	GtkWidget *text_entry=glade_xml_get_widget(xml,"combo-entry");
	G_CONST_RETURN gchar *text=gtk_editable_get_chars(GTK_EDITABLE(text_entry),0,-1);
	vte_terminal_feed_child(VTE_TERMINAL(vte), text, strlen(text));
}

/*************************************************************************************************/
void on_fit_button_clicked(GtkWidget *widget, gpointer user_data) {
	create_fit_options(vte);
}

/*************************************************************************************************/
void on_eps_button_clicked(GtkWidget *widget, gpointer user_data) {
	gchar *text="set term post eps enhanced color solid \"Times-Roman\" 12\n"
		 "set output \"|ggv -\"\n"
		 "replot\n"
		 "set output \"|cat ->/dev/null\"\n"
		 "set term x11\n";
	vte_terminal_feed_child(VTE_TERMINAL(vte), text, strlen(text));
}

/*************************************************************************************************/
void on_combo_list_select_child(GtkList *list,GtkWidget *widget,gpointer user_data) {
	/*static gboolean ok=TRUE;
	ok=!ok;
	if(widget==NULL || ok ) return;
	G_CONST_RETURN gchar *text=gtk_label_get_text(GTK_LABEL(GTK_BIN(widget)->child));
	vte_terminal_feed_child(VTE_TERMINAL(vte), text, strlen(text));*/
}

/*************************************************************************************************/
void connect_vscrool_with_vte(GladeXML *xml) {
	VteTerminal *terminal=VTE_TERMINAL(vte);
	GtkWidget *vscrollbar=glade_xml_get_widget(xml,"vscrollbar");
	gtk_range_set_adjustment(GTK_RANGE(vscrollbar),terminal->adjustment);
}

/*************************************************************************************************/
int main(int argc, char *argv[]) {
    //GladeXML *xml;
    GError *error;
    
    gtk_init(&argc, &argv);
    glade_init();

    /* load the interface */
    /*xml = glade_xml_new(GLADE_MAIN, NULL, NULL);*/
    xml = glade_xml_new_from_buffer( main_xml, strlen(main_xml), NULL, NULL );
    
    if(xml==NULL) {
    	printf("Error: gtk_gnuplot.glade not found.\n\n");
    	exit(0);
    }
    
    vte_new();

    /* connect the signals in the interface */
    glade_xml_signal_autoconnect(xml);
    
    /* connect the VScrool with vte*/
    connect_vscrool_with_vte(xml);

    /* start the event loop */
    gtk_main();

    return 0;
}

